package oracle.ugbu.ss.portal.backing;

//import com.oracle.ugbu.ss.api.model.security.OUCSSCommercialUser;


import com.oracle.ugbu.ss.api.model.account.OUCSSAccount;
import com.oracle.ugbu.ss.api.model.account.OUCSSAccountContextInfo;
import com.oracle.ugbu.ss.api.model.common.OUCSSConstants;
import com.oracle.ugbu.ss.api.model.common.OUCSSList;
import com.oracle.ugbu.ss.api.model.context.OUCSSCommercialContext;
import com.oracle.ugbu.ss.api.model.context.OUCSSContext;
import com.oracle.ugbu.ss.api.model.factory.OUCSSAbstractFactory;
import com.oracle.ugbu.ss.api.model.security.OUCSSUser;
import com.oracle.ugbu.ss.common.SSUtil;
import com.oracle.ugbu.ss.shared.model.cache.OUCSSCacheManager;
import com.oracle.ugbu.ss.shared.model.common.OUCSSModelUtil;
import com.oracle.ugbu.ss.shared.model.factory.OUCSSFactoryImpl;
import com.oracle.ugbu.ss.shared.model.helper.OUCSSSessionHelperImpl;

import java.io.Serializable;

import java.util.Map;

import javax.faces.context.FacesContext;

import javax.servlet.http.HttpServletRequest;

import javax.servlet.http.HttpServletResponse;

import oracle.adf.share.ADFContext;
import oracle.adf.share.logging.ADFLogger;

/**
 * Class to manage the OUCSS related context in User Session.
 */
public class OUCSSSessionBean
  implements Serializable
{
  @SuppressWarnings("compatibility")
  private static final long serialVersionUID = 1L;
  private static final ADFLogger _LOG =
    ADFLogger.createADFLogger(OUCSSSessionBean.class);
  private static final String DEFAULT_LANDING_PAGE_CD = "DEFAULT";
  private static Map<String, String> LOB_LANDING_PAGE_MAP;
  private static final String PORTAL_LANDING_PAGE_CD =
    "PORTAL_LANDING_PAGE";

  private Boolean mValidUser = null;
  private Boolean mEnrolledUser = null;
  private Boolean mCSRUser = null;
  private OUCSSContext<OUCSSUser, OUCSSAccount> mDefaultContext;
  private String mContextLOB = null;

  /**
   * Method to check if user is valid OCUSS User. The user is declared valid if
   * member isCSRUser is true.
   * @return
   */
  public boolean isValidOUCSSUser()
  {
    if (mValidUser == null)
    {
      mValidUser = Boolean.FALSE;
      if (ADFContext.getCurrent().getSecurityContext().isAuthenticated())
      {
        // Check if user is OUCSS valid user.
        mValidUser = getContext().isValidOUCSSUser();

        //Validate user if user is member of either WSSCSR or WSSAdmin Group.
        if (!mValidUser && isCSRUser())
        {
          mValidUser = Boolean.TRUE;
        }
      }
    }
    return mValidUser.booleanValue();
  }

  /**
   * Method to check if user has enrolled or invited accounts.
   * @return
   */
  public boolean isEnrolledUser()
  {
    if (mEnrolledUser == null)
    {
      mEnrolledUser = Boolean.FALSE;
      if (isValidOUCSSUser())
      {
        mEnrolledUser = getContext().isEnrolledOUCSSUser();
        if (!mEnrolledUser && isCSRUser())
        {
          mEnrolledUser = Boolean.TRUE;
        }
      }
    }
    return mEnrolledUser.booleanValue();
  }

  /**
   * Method to check if the account is prepaid or regular account.
   * @return true if the account if of prepaid type, false otherwise.
   */
  public String getAccountType()
  {
    String accountType = "UNKNOWN";
    if (isEnrolledUser())
    {
      //Get the Account Context for current account.
      try
      {
        OUCSSAccountContextInfo accountContext =
          getContext().getCurrentUserAccount().getAccountContextInfo();
        if (accountContext != null)
        {
          //Determine if the account is prepaid or not.
          boolean isPrepaid = accountContext.isPrePaid();
          if (isPrepaid)
          {
            accountType = "PREPAID";
          }
          else
          {
            accountType = "POSTPAID";
          }
        }
      }
      catch (Exception e)
      {
        //Log exception if unable to determine the account type.
        _LOG.warning(e);
      }
    }
    return accountType;
  }

  /**
   * Method to check if logged in user is CSR or Admin
   * @return true if user is member of WSSCSRGroup or WSSAdminGroup, false otherwise.
   */
  protected boolean isCSRUser()
  {
    if (mCSRUser == null)
    {
      mCSRUser = OUCSSModelUtil.isCSRUser();
    }
    return mCSRUser.booleanValue();
  }

  /**
   * Method to get the Context. Context sets up the user and user accounts on demand.
   * @return
   */
  public OUCSSContext<OUCSSUser, OUCSSAccount> getContext()
  {
    if (mDefaultContext == null)
    {
      String lobEaId = getContextLOB();

      //Create context with null user as Context will evaluate the context user id.
      mDefaultContext =
          getAbstractFactory(lobEaId).createContext(null, lobEaId);
    }
    return mDefaultContext;
  }

  public OUCSSList<OUCSSAccount> getMultipleAccounts()
  {
    OUCSSList<OUCSSAccount> mSelectedAccounts =
      ((OUCSSCommercialContext) getContext()).getBusinessContext().getSelectedAccounts();

    return mSelectedAccounts;
  }

  public OUCSSList<OUCSSAccount> getSelectedAccount()
  {
    String lobEaId = getContextLOB();

    OUCSSList<OUCSSAccount> mSelectedAccounts =
      getAbstractFactory(lobEaId).createAccountList();

    mSelectedAccounts.add(getContext().getCurrentUserAccount());

    return mSelectedAccounts;
  }

  public String getLandingPage()
  {
    String returnPage = null;
    String lobEaId = getContextLOB();

    if (lobEaId != null)
    {
      String lob =
        OUCSSCacheManager.getLOBCache().getItemAsString(lobEaId, SSUtil.getUserLocaleLanguage());
      if (isSingleEnrolledAccount())
      {
        lob = lob + "_SA";
      }
      returnPage = getLOBLandingPageMap().get(lob);
    }

    if (returnPage == null)
    {
      returnPage = getLOBLandingPageMap().get(DEFAULT_LANDING_PAGE_CD);
    }
    return returnPage;
  }

  /**
   * Method to check if the user is enrolled to only one account (for logged in lob).
   * @return true if user has one account enrolled, false otherwise.
   */
  private boolean isSingleEnrolledAccount()
  {
    boolean singleAccount = false;
    if (isEnrolledUser())
    {
      singleAccount =
          getContext().getCurrentUserAccountList().getSize() == 1;
    }
    return singleAccount;
  }

  private static Map<String, String> getLOBLandingPageMap()
  {
    if (LOB_LANDING_PAGE_MAP == null || LOB_LANDING_PAGE_MAP.isEmpty())
    {
      LOB_LANDING_PAGE_MAP =
          SSUtil.getLOBSiteResource(PORTAL_LANDING_PAGE_CD);
    }
    return LOB_LANDING_PAGE_MAP;
  }

  private OUCSSAbstractFactory<OUCSSUser, OUCSSAccount> getAbstractFactory(String lobEaId)
  {
    return OUCSSFactoryImpl.getInstance().getFactory(lobEaId);
  }

  /**
   * Method to evaluate Context LOB. If not found, then set to default lob.
   * @return
   */
  public String getContextLOB()
  {
    //As LOB is null, set it to residential by default.
    if (mContextLOB == null)
    {
      mContextLOB =
          (String) OUCSSSessionHelperImpl.getInstance().getSessionData(OUCSSConstants.OUCSS_USER_LOB_INFO_KEY);
      
      if (mContextLOB == null)
      {
        mContextLOB =  OUCSSConstants.OUCSS_DEFAULT_LOB;
        //Add the LOB EA Id to session scope.
        OUCSSSessionHelperImpl.getInstance().setSessionData(OUCSSConstants.OUCSS_USER_LOB_INFO_KEY,
                                                            mContextLOB);
      }
    }
    return mContextLOB;
  }
}
